/* Arduino Wiggler Cable Simulator */
/* Intended for use with tjtag3-0-1 with JTMOD patch. */


#define WRSTN 34  //C2
#define WTMS 35   //C3
#define WTCK 36   //C4
#define WTDI 37   //C5
#define WTRSTN 38 //C6
#define WTDO 39   //C7

#define PIN_MASK (g_APinDescription[WRSTN].ulPin | g_APinDescription[WTMS].ulPin | g_APinDescription[WTCK].ulPin | g_APinDescription[WTDI].ulPin | g_APinDescription[WTRSTN].ulPin)
#define PIN_PORT g_APinDescription[WRSTN].pPort // Digital Pins 34-39 are port C on the Arduino Due

typedef
enum __arduOp
{
    OP_RESET = 0,
    OP_SEND  = 1,
    OP_READ  = 2,
    OP_RSVD  = 3
}
arduOp;                                                                                                                                                      

void pinOutputLow(uint32_t ulPin)
{
   PIO_Configure( g_APinDescription[ulPin].pPort, PIO_OUTPUT_0, g_APinDescription[ulPin].ulPin, g_APinDescription[ulPin].ulPinConfiguration ) ;
}

void setup(void)
{
    // Pins 0-7 are part of PORTD
    // pins 0 and 1 are RX and TX, respectively

    PIN_PORT->PIO_WPMR = PIN_MASK; // Disable Write Protection on Pins
    PIN_PORT->PIO_OWER = PIN_MASK; // Enable Writing to All pins at once
  
    pinOutputLow(WRSTN);    // WRST_N
    pinOutputLow(WTMS);     // WTMS
    pinOutputLow(WTCK);     // WTCK
    pinOutputLow(WTDI);     // WTDI
    pinOutputLow(WTRSTN);   // WTRST_N

    pinMode(WTDO, INPUT);      // WTDO

    //Serial.begin(9600);
    Serial.begin(115200);

    /*
     * Before starting...
     * Say something invalid so that if the Arduino
     * gets reset in the middle of a run, the tjtag
     * program will know that something's gone wrong.
     *
     * This is invalid because our simple protocol
     * says that bits 6-0 should never be set except
     * when responding to a reset command with 0x42.
     */
    Serial.write(0x7F);
}

void loop(void)
{
    // Wait until byte available.
    while ( ! Serial.available() );

    int value = Serial.read();
    unsigned char pcin = value;

    //Serial.print("byte received: ");
    //Serial.println(pcin, HEX);

    unsigned char op = (pcin & B01100000) >> 5 ;

    //Serial.print("opcode interpreted: ");
    //Serial.println(op, HEX);


    switch ( op )
    {
        case OP_RESET:
            {
                //Serial.println("  OP_RESET");
                // Clear all outputs.
                digitalWrite(WRSTN, LOW);
                digitalWrite(WTMS, LOW);
                digitalWrite(WTCK, LOW);
                digitalWrite(WTDI, LOW);
                digitalWrite(WTRSTN, LOW);

                // Respond that we're ready to go.
                Serial.write(0x42);

                // Clear out any other incoming data.
                Serial.flush();
            }
            break;
            
        case OP_SEND:
            {
                //Serial.println("  OP_SEND");

                // Set output pins to requested values.
                // PORTD = (unsigned char)( (pcin & B00011111) << 2 );
                PIN_PORT->PIO_ODSR = (PIN_PORT->PIO_ODSR & ~(PIN_MASK)) | ((pcin & B00011111) << 2 );
                // If needed:  put a Serial.write() here so that
                // we can "wait" for the pins to be set.
                Serial.write(0x4B);
            }
            break;

        case OP_READ:
            {
                //Serial.println("  OP_READ");

                unsigned char readByte = digitalRead(WTDO) == HIGH ?     // WTDO
                                            B10000000 :     // send back a 1 in bit 7
                                            B00000000;      // send back a 0 in bit 7

                Serial.write(readByte);
            }
            break;

        default:
            {
                // BAD OPCODE
                // Send invalid data in response.
                Serial.write(0x7F);
            }
    };

    //Serial.println();
    //Serial.println();
}
