/*Done by J. Trimble.
 *Modified by David Teran Garrido (davidtgbe@hotmail.com)
 *Little reminder: This mod has been programmed for personal use
 *to debrick my WRT54G and is not fully tested so I'm not responsible of possible bugs.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <windows.h>
#include "jt_mods.h"
#include "rs232.h"

/***************************************
 * (CHANGE) Port number and baud rate
 * Must be the same config on arduino
 ****************************************/

#define COMPORT 4
#define BAUDRATE 115200

// Set to non-zero to enable debug.
int jtMod_debug = 0;

#define portIndex (COMPORT - 1)

/***********************
 * Copied from tjtag.h:
 ***********************/
// --- Xilinx Type Cable ---

#define TDI     0
#define TCK     1
#define TMS     2
#define TDO     4

// ---Wiggler Type Cable ---

#define WTDI      3
#define WTCK      2
#define WTMS      1
#define WTDO      7
#define WTRST_N   4
#define WSRST_N   0
/***********************/

#define sleep(n) Sleep(1000 * n)

typedef
enum __arduOp
{
    OP_RESET = 0,
    OP_SEND  = 1,
    OP_READ  = 2,
    OP_RSVD  = 3
}
arduOp;

#define ARDUOP_BITS (0x60) // 0110 0000
#define ARDUOP_LSHIFT (5)
#define ARDUOP_RESET  (OP_RESET << ARDUOP_LSHIFT) // currently zero...
#define ARDUOP_READ   (OP_READ  << ARDUOP_LSHIFT) // currently B0100 0000

#define ASSERT(expr) ASSERT_WFL(expr, __FILE__, __LINE__)

#define ASSERT_WFL(expr, file, line)                \
if ( !(expr) )                                      \
{                                                   \
    fprintf(stderr,                                 \
            "jtMod assert failed (%s:%d): '%s'\n",  \
            (file), (line),                         \
            (#expr) );                              \
                                                    \
    exit(42);                                       \
}

int jtMod_init(void)
{

    if (OpenComport(portIndex,BAUDRATE))
    {
        char buf[64];
        snprintf(buf, 64, "Failed to open COM'%d'",
                 COMPORT);
        buf[63] = '\0';
        perror(buf);
        exit(42);
    }

    printf("JTMOD: Opened port.\n");

    printf("JTMOD: Resetting Arduino.\n");

    // Send reset signal to Arduino,
    {
        unsigned char msg = ARDUOP_RESET;
        SendBuf(portIndex,&msg,1);
    }

    {
        int tries = 0;
        while (1)
        {
            // wait for response before continuing.
            unsigned char msg;
            int bytesRead;
            bytesRead = PollComport(portIndex,&msg,1);

            if (jtMod_debug && bytesRead > 0)
                printf("JTMOD:    Waiting...read %d bytes, 0x%X\n", bytesRead, msg);

            if ( bytesRead > 0 &&  msg == 0x42 )
                break;

            if (++tries > 10)
            {
                fprintf(stderr, "JTMOD:  Hmm.  Looks like the Arduino isn't responding.  Did you program it with the correct sketch?\n");
                exit(42);
            }
            else
            {
                sleep(1);
            }
        }

    }

    printf("JTMOD: Ready.\n");
    return 0;
}

int jtMod_inp (unsigned char * p_byteToFill)
{
    // Read byte from port.
    unsigned char msg = ARDUOP_READ;
    int    bytesRead;
    int    tries;

    if (jtMod_debug)
        printf("\nJTMOD:  Entering jtMod_inp\n");

    SendBuf(portIndex,&msg,1);

    tries = 0;
    bytesRead = -1;
    while (bytesRead <= 0 && tries < 2)
    {
    	bytesRead = PollComport(portIndex,&msg,1);
        if (jtMod_debug)
        	printf("JTMOD:  Recibido 0x%X\n",msg);

        tries++;
    }

    if (bytesRead < 0)
    {
        char buf[128];
        snprintf(buf, 128, "read from Arduino returned %d: ", bytesRead);
        buf[127] = '\0';
        perror(buf);
        exit(42);
    }

     //None of bits 6-0 should be set.
    if ( msg & 0x7F )
    {
        fprintf(stderr, "Invalid data (%#X) on read from Arduino!%\n", msg);
        exit(42);
    }

    *p_byteToFill = msg;
    *p_byteToFill ^= 0x80;      // it took me hours to find this

    if (jtMod_debug)
        printf("JTMOD:  Leaving jtMod_inp\n");

    return 0;
}

int jtMod_outp(unsigned char p_byteToSend)
{
    // Pack bits into port.
    unsigned char msg;
    int bytesRead;

    if (jtMod_debug)
        printf("\nJTMOD:  Entering jtMod_outp\n");

    // Populate bits 4,3,2,1,0
    msg = p_byteToSend;

    // Clear bits 7,6,5
    msg &= 0x1F;        // B0001 1111

    // Set bits 6,5 to '01'
    msg |= 0x20;        // B0010 0000

    // Send to Arduino
    if (jtMod_debug)
        printf("JTMOD: Enviando... 0x%X\n",msg);

    SendBuf(portIndex,&msg,1);

    // Wait for response before continuing.
    bytesRead = -1;
    while (bytesRead <= 0)
    {
    	bytesRead = PollComport(portIndex,&msg,1);
    }
    if (jtMod_debug)
    	printf("JTMOD:  Recibido 0x%X\n",msg);

    if ( bytesRead != 1 || msg != 0x4B )
    {
        fprintf(stderr, "Invalid data (0x%X) while waiting for send to complete!\n", msg);
        exit(42);
    }

    if (jtMod_debug)
        printf("JTMOD:  Leaving jtMod_outp\n");

    return 0;
}

int jtMod_done(void)
{
    printf("JTMOD: Cleaning up.\n");

    // Send reset signal to Arduino,
    {
        unsigned char msg = ARDUOP_RESET;
        SendBuf(portIndex,&msg,1);
    }

    // Reset and close port.

    CloseComport(portIndex);
    printf("JTMOD: Done.\n");
    return 0;
}
